/*
 *    Poller.java
 *    Copyright (C) 2000 Stuart Yeates 
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.ptp;

// java standard library classes used
import java.io.IOException;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.Writer;
import java.io.OutputStreamWriter;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Serializable;
//import java.io.;
import java.util.*;
import java.net.*;

/**
 * Class Poller polls a group of peers periodically. 
 *
 * Poller exists in it's  own Thread so it is asynchronous with 
 * the rest of the program. The Connections to peers are also in
 * seperate Threads, but only one run at once.
 *
 * The static Node.knownList is consulted about which peers to 
 * poll.
 *
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2119 $
 * @see org.nzdl.gsdl.ptp.PTP
 * @see org.nzdl.gsdl.ptp.Connection
 * @see org.nzdl.gsdl.ptp.Node
 * @see org.nzdl.gsdl.ptp.Node#knownList
 * @see java.lang.Thread
 */

public class Poller extends Thread {
  /** How often should we poll our peers? */
  int secondsBetweenPollCycles = 60; 
  
  /** The no-args constructor */
  public Poller() {

  }

  /** 
   * Start the poller. 
   * 
   * Only returns if interrupted or an Error is thrown.
   */
  public void run() {
    while (true) {
      poll();
      try {
	sleep(secondsBetweenPollCycles * 1000);
      } catch (InterruptedException exception) {
	System.err.println("caught exception: "+exception);
	return;
      }
    }
  }

  /** 
   * Poll each peer once, waiting for the Connections to terminate
   * between each. 
   * 
   * Returns when all peer has been polled.
   */
 void poll() {
    Enumeration e = Node.knownList.propertyNames();
    while (e.hasMoreElements()) {
      String next = (String) e.nextElement();
      PTP ptp = (PTP) Locator.parse(next);
	System.err.println("About to create new thread for " + next);
	Connection connection = 
	  new Connection(ptp.host,ptp.port);
	System.err.println("About to start new thread");
	connection.run();
	System.err.println("About to wait for new thread");
	try {
	  connection.join();
	} catch (InterruptedException exception) {
	  System.err.println("caught exception: "+ exception);
	  return;
	}
	System.err.println("Joined with new thread");
    }	
    
  }
  

}





